@@ -14,6 +14,16 @@ from shapely.impl import DefaultImplementation, delegated
 
 if sys.version_info[0] < 3:
     range = xrange
+    integer_types = (int, long)
+else:
+    integer_types = (int,)
+
+try:
+    import numpy as np
+    integer_types = integer_types + (np.integer,)
+except ImportError:
+    pass
+
 
 GEOMETRY_TYPES = [
     'Point',
@@ -63,7 +73,7 @@ def geom_factory(g, parent=None):
         [geom_type],
         )
     ob.__class__ = getattr(mod, geom_type)
-    ob.__geom__ = g
+    ob._geom = g
     ob.__p__ = parent
     if lgeos.methods['has_z'](g):
         ob._ndim = 3
@@ -188,14 +198,11 @@ class BaseGeometry(object):
     _ndim = None
     _crs = None
     _other_owned = False
+    _is_empty = True
 
     # Backend config
     impl = DefaultImplementation
 
-    @property
-    def _is_empty(self):
-        return self.__geom__ in [EMPTY, None]
-
     # a reference to the so/dll proxy to preserve access during clean up
     _lgeos = lgeos
 
@@ -207,6 +214,7 @@ class BaseGeometry(object):
                 self._lgeos.GEOSGeom_destroy(self.__geom__)
             except AttributeError:
                 pass  # _lgeos might be empty on shutdown
+        self._is_empty = True
         self.__geom__ = val
 
     def __del__(self):
@@ -235,6 +243,7 @@ class BaseGeometry(object):
     @_geom.setter
     def _geom(self, val):
         self.empty()
+        self._is_empty = val in [EMPTY, None]
         self.__geom__ = val
 
     # Operators
@@ -528,7 +537,7 @@ class BaseGeometry(object):
 
     @delegated
     def simplify(self, tolerance, preserve_topology=True):
-        """Returns a simplified geometry produced by the Douglas-Puecker
+        """Returns a simplified geometry produced by the Douglas-Peucker
         algorithm
 
         Coordinates of the simplified geometry will be no more than the
@@ -663,6 +672,12 @@ class BaseGeometry(object):
         specified decimal place"""
         return self.equals_exact(other, 0.5 * 10**(-decimal))
 
+    def relate_pattern(self, other, pattern):
+        """Returns True if the DE-9IM string code for the relationship between
+        the geometries satisfies the pattern, else False"""
+        pattern = c_char_p(pattern.encode('ascii'))
+        return bool(self.impl['relate_pattern'](self, other, pattern))
+
     # Linear referencing
     # ------------------
 
@@ -828,7 +843,7 @@ class GeometrySequence(object):
     def __getitem__(self, key):
         self._update()
         m = self.__len__()
-        if isinstance(key, int):
+        if isinstance(key, integer_types):
             if key + m < 0 or key >= m:
                 raise IndexError("index out of range")
             if key < 0:
