@@ -1,13 +1,14 @@
 from . import unittest
 
 from shapely.libgeos import load_dll
+import sys
 
 
 class LoadingTestCase(unittest.TestCase):
 
     def test_load(self):
         self.assertRaises(OSError, load_dll, 'geosh_c')
-
+    @unittest.skipIf(sys.platform == "win32", "FIXME: adapt test for win32")
     def test_fallbacks(self):
         load_dll('geos_c', fallbacks=[
             '/opt/local/lib/libgeos_c.dylib',  # MacPorts
